%--------------------------------------------------------------------------
% BAYESIAN COMPUTATION OF POSTERIOR CONFIDENCE BANDS FOR 
% FOR CONJUGATE NORMAL-INVERSE WISHART PRIOR
% LEE SMITH
% CODE IS BASED ON GARY KOOP AND DIMITRIS KOROBILIS'S CODE FROM THEIR PAPER
% "BAYESIAN MULTIVARIATE TIME SERIES METHODS FOR EMPIRICAL MACROECONOMICS"
%--------------------------------------------------------------------------


function [IRpoint, allIRs, VARshocks] = confid_int_exo(Bcomp,VC_eps,cvec,isexo,IRhoriz,NMC,Y,expl,A_prior,a_prior,V_prior,v_prior,S_prior, shock_size);

%--------------------------------------------------------------------------
%
% DESCRIPTION:
%
% This function computes the impulse response functions and confidence intervals
%
%--------------------------------------------------------------------------
%
% INPUTS:
%
% Bcomp:   matrix with structure of estimated reduced-form coefficients in 
%           companion form excluding all the deterministic terms. 
%           This includes the estimated parameters in the partition 
%           n x (n x (n x vlag)). The remaining partition is made up of a 
%           diagonalic identity matrix that pins down the lead-lag relation 
%           between variables. The size is (n x vlag) x (n x vlag).     
%
% VC_eps:  covariance matrix of reduced-form residuals, size n x n.
%
% cvec:    matrix with estimated constants; this returns the scalar 0 
%           if no constant is included in the model; 
%           otherwise, this is a matrix of size n x 1.
%
% isexo:   Non-empty matrix representing dummy variable/controls
%
% IRhoriz: number of periods for which impulse responses are computed
%
% NMC:     number of Monte Carlo simulations for the density of simulated
%           impulse responses
%
% Y:       Raw Data
%
% CIperc:  this is a 2 x 1 vector with lower and upper percentiles for 
%           the confidence itervals that are computed and, eventually,
%           plotted
%
% expl:	   accept/reject explosive draws (0 accept, 1 reject)
%
% A_prior: Prior hypermatrix governing prior mean for VAR coefficients Nbig*(Nbig*vlag + 1) x Nbig
%
% a_prior: Prior hypervector governing prior mean for VAR coefficients Nbig*(Nbig*(Nbig*vlag + 1)) x 1 (i.e. a_prior = vec(A_prior))
%
% V_prior: Prior hypermatrix governing variance of prior mean (A_prior) Nbig*(Nbig*vlag + 1) x Nbig*(Nbig*vlag + 1)
%
% v_prior: Prior hyperparameter governing degrees of freedom of prior covariance matrix of VAR residuals
%
% S_prior: Prior hypermatrix governing scale of prior covariance matrix of VAR residuals
%--------------------------------------------------------------------------
%
% OUTPUT:
%
% IRpoint: point estimate of impulse responses 
%
% IRup:    upper bound of impulse responses 
%
% IRlo:    lower bound of impulse responses
%
% IRmed:   median of impulse response distribution
%
% VARshocks: structural shocks at posterior mean
%
%%----------------------------------------------------------------------------------------------------------------------------------
%% BACK OUT THE SIZE OF VARIOUS OBJECTS OF INTEREST
[Tbig,Nbig] = size(Y);
iscon  		= sum(abs(cvec))>0;
Nbig   		= length(VC_eps);
Nbigcomp 	= length(Bcomp);
vlag   		= length(Bcomp)/Nbig;

%% INITIALIZE STORAGE
allIRs = NaN(IRhoriz+1, Nbig, Nbig, NMC);

%% CREATE OLS MATRICES
BIG_Y 		= Y(vlag+1:Tbig,:);
BIG_X 		= zeros(Tbig-vlag,Nbig*vlag);

for lags = 1:1:vlag;
	lower_index_row		= vlag+1-lags;
	upper_index_row		= Tbig-lags;
	upper_index_col		= Nbig*lags;
	lower_index_col		= upper_index_col - Nbig + 1;
	BIG_X(:,lower_index_col:upper_index_col) = Y(lower_index_row:upper_index_row,:); 	
end;

if iscon == 1
	BIG_X = [BIG_X, ones(Tbig-vlag,1)];
end;

if ~isempty(isexo);
	isexo = isexo(:,any(isexo(vlag+1:end,:),1));
    BIG_X = [BIG_X, isexo(vlag+1:end, :)];
end

%%----------------------------------------------------------------------------------------------------------------------------------
%% CALCULATE POSTERIOR DISTRIBUTION OF VAR PARAMETERS
A_OLS 	= (BIG_X'*BIG_X)\BIG_X'*BIG_Y; 
wishdof = Tbig;
scale	= Tbig - vlag - 1;
SSE		= scale*VC_eps;
% VC_eps is calcualted using MATLAB's 
% cov() function which divides the SSE by (Tbig-vlag-1).
% Therefore, 'scale=Tbig-vlag-1' undoes this so that we 
% are working with the raw SSE

% For alpha
V_post = inv(inv(V_prior) + (BIG_X'*BIG_X));
A_post = (inv(V_prior)+(BIG_X'*BIG_X))\(inv(V_prior)*A_prior + (BIG_X'*BIG_X)*A_OLS);
a_post = reshape(A_post,Nbig*(Nbig*vlag+iscon+size(isexo,2)),1);

% For SIGMA
C_S_post = chol(SSE + S_prior + A_OLS'*(BIG_X'*BIG_X)*A_OLS + A_prior'*inv(V_prior)*A_prior - A_post'*(inv(V_prior)+(BIG_X'*BIG_X))*A_post)';
S_post   = C_S_post*C_S_post';
v_post   = Tbig - vlag + v_prior;
[ans,DI] = iwishrnd(S_post,v_post); % THIS SPEEDS-UP DRAWS

%%----------------------------------------------------------------------------------------------------------------------------------
% CALCUALTE IRFS AT POSTERIOR MEAN
Bcomp_post	= [A_post(1:Nbig*vlag,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
			   zeros((vlag-1)*Nbig, Nbig)];
IRpoint		= VAR_irf(Bcomp_post, S_post/(v_post-1), IRhoriz);
Respoint 	= BIG_Y - BIG_X*A_post;
Cholpoint 	= chol(S_post/(v_post-1))';
STRshocks  	= Respoint*inv(Cholpoint)'; 
VARshocks	= STRshocks;

% Tolerance for eigenvalue calculation
opts.tol = 1e-2; 
e_0		 = norm(eigs(Bcomp_post,1,'lm',opts));

%% DO MC RUNS TO GET POSTERIOR PROBABILITY INTERVALS
for mm=1:NMC;
	draw = mm;
	workbar(mm/NMC,'Performing Monte Carlo Integration...','Progress');
	%% HERE I EMPLOY ANTITHETIC SAMPLING WHICH SPEEDS UP THE CONVERGENCE
	if mod(mm,2) == 1
		draw_again = 1;
		while draw_again == 1;
			% odd draw again
			VC_eps_draw 		= iwishrnd(S_post, v_post,DI);
			vec_BIG_A_COVAR		= kron(VC_eps_draw, V_post);
            vec_BIG_A_COVAR     = (vec_BIG_A_COVAR + vec_BIG_A_COVAR')/2;  %Necessary on Mac OS only
			vec_A_draw_u		= mvnrnd(zeros(Nbig*(Nbig*vlag+iscon+size(isexo,2)),1)',vec_BIG_A_COVAR);
			vec_BIG_A_draw		= a_post + vec_A_draw_u'; 
			if expl == 1
				BIG_A_draw			= reshape(vec_BIG_A_draw,Nbig*vlag+iscon+size(isexo,2),Nbig);
				Bcomp_draw_1		=[BIG_A_draw(1:Nbig*vlag,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
										zeros((vlag-1)*Nbig, Nbig)];
				norm_1				= norm(eigs(Bcomp_draw_1,1,'lm',opts));
				vec_BIG_A_draw_2	= a_post - vec_A_draw_u'; 
				BIG_A_draw_2		= reshape(vec_BIG_A_draw_2,Nbig*vlag+iscon+size(isexo,2),Nbig);
				Bcomp_draw_2		=[BIG_A_draw_2(1:Nbig*vlag,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
										zeros((vlag-1)*Nbig, Nbig)];
				norm_2				= norm(eigs(Bcomp_draw_2,1,'lm',opts));
				if norm_1 < 1.0+opts.tol && norm_2 < 1.0+opts.tol;
					draw_again = 0;
				end;
			elseif expl == 0
				draw_again = 0;
			end;
		end
	else
		% even draw - take reflection
		vec_BIG_A_draw		= a_post - vec_A_draw_u'; 
	end;
	%%
	BIG_A_draw			= reshape(vec_BIG_A_draw,Nbig*vlag+iscon+size(isexo,2),Nbig);
	Bcomp_draw			=[BIG_A_draw(1:Nbig*vlag,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
									zeros((vlag-1)*Nbig, Nbig)];
	allIRs(:, :, :, mm) = VAR_irf(Bcomp_draw, VC_eps_draw, IRhoriz);   
	
end